#include "StdAfx.h"
#include ".\wtextfile.h"

WTextFile::WTextFile( wchar_t *filename, BOOL checkCRs, BOOL checkNewLines, BOOL checkVertTabs,
	BOOL checkNewPages, BOOL checkTabs, BOOL checkBlanks )
: m_actualLine( 0 )
, m_numberOfLines( 0 )
, m_numberOfCols( 0 )

, m_checkCRs( true )
, m_checkNewLines( false )
, m_checkVertTabs( false )
, m_checkNewPages( false )
, m_checkTabs ( true )
, m_checkBlanks( false )
{
	
	m_checkCRs = checkCRs;
	m_checkNewLines = checkNewLines;
	m_checkVertTabs = checkVertTabs;
	m_checkNewPages = checkNewPages;
	m_checkTabs = checkTabs;
	m_checkBlanks = checkBlanks;
	
	m_linebuffer = new wstring( L"" );
	m_file = _wfopen( filename, L"r" );

	///////////////init wordArray////////////////////////
	countLinesCols();
	fseek( m_file, 0L, SEEK_SET );

	m_wordArray = new wstring**[m_numberOfLines]; // rows

	for (int i=0; i < m_numberOfLines; i++)
	{
		m_wordArray[i] = new wstring*[m_numberOfCols+1]; // columns
	}

	for (int i=0; i < m_numberOfLines; i++)
	{
		for (int j=0; j < m_numberOfCols+1; j++)
		{
			m_wordArray[i][j] = new wstring( L"" );
		}
	}
	///////////////////////////////////////////////////////////////
}

WTextFile::~WTextFile( void )
{
	delete m_linebuffer;

	for (int i=0; i < m_numberOfLines; i++)
	{
		for (int j=0; j < m_numberOfCols+1; j++)
		{
			delete m_wordArray[i][j];
		}
	}

	for (int i=0; i < m_numberOfLines; i++)
	{
		delete[] m_wordArray[i]; // columns
	}

	delete[] m_wordArray;

	fclose( m_file );
}

void WTextFile::readFile( )
{
	while( (readln( m_linebuffer )) != WEOF )
	{
		putLineInWordArray( );
	}

	// Internally ART always uses a point;
	// eventually replace ',' by '.'
	for( int row = 0; row < m_numberOfLines; row++ )
	{
		for( int column = 0; column < m_numberOfCols; column++ )
		{
			replaceCommaDot( m_wordArray[row][column] );
		}
	}
}

BOOL WTextFile::replaceCommaDot( wstring *buffer )
{
	for( int i=0; i < buffer->size(); i++ )
	{
		if( buffer->at( i ) == L',' )
			buffer->at( i ) = L'.';
	}

	return false;
}

wstring **WTextFile::returnWordArray( )
{
	return *m_wordArray;
}

wstring *WTextFile::at( int line, int column )
{
	return m_wordArray[line][column];
}

int WTextFile::readln( wstring *buffer )
{
	buffer->clear();
	wchar_t theChar;
	
	while ( ( ( theChar = getwc( m_file ) ) != WEOF ) && ( ! checkLineDelimiter( theChar ) ) )
	{
		*buffer += theChar;
	}

	
	if( buffer->length() > 0 )
		return buffer->length();
	else
		if( buffer->length() == 0 )
		{
			if( theChar == WEOF )
				return WEOF;
			else
				return 0;
		}

		return -1;
}

void WTextFile::putLineInWordArray( )
{
	int wordcounter = 0;
	int length = m_linebuffer->length();
	int currentPos = 0;
	wstring *word;
	wstring term = L"\r";

	while (currentPos < length)
	{
		while ((checkWordDelimiter (m_linebuffer->at( currentPos ))) && (++currentPos < length ))
		{
		}
		currentPos--;

		word = m_wordArray[m_actualLine][wordcounter];

		while ( ( ++currentPos < length ) && (!checkWordDelimiter( m_linebuffer->at(currentPos) ) ) )
		{
			*word += m_linebuffer->at( currentPos );
		}
		
		wordcounter++;
	}

	*m_wordArray[m_actualLine][wordcounter] = term;
	m_actualLine++;
}

BOOL WTextFile::checkWordDelimiter(wchar_t character)
{
	wchar_t delimiters[6];

	delimiters[0] = '\n';
	delimiters[1] = '\r';
	delimiters[2] = '\v';
	delimiters[3] = '\f';
	
	for (int i=0; i < 4; i++)
	{
		if (character == delimiters[i])
			return true;
	}

	if( m_checkTabs )
	{
		if( character == '\t' )
			return true;
	}

	if( m_checkBlanks )
	{
		if( character == ' ' )
			return true;
	}
	
	return false;
}

BOOL WTextFile::checkLineDelimiter(wchar_t character)
{
	if( m_checkCRs )
	{
		if (character == '\r')
			return true;
	}

	if( m_checkNewLines )
	{
		if (character == '\n')
			return true;
	}

	if( m_checkVertTabs )
	{
		if (character == '\v')
			return true;
	}

	if( m_checkNewPages )
	{
		if (character == '\f')
			return true;
	}

	return false;
}

void WTextFile::countLinesCols(void)
{
	m_numberOfLines = 0;
	m_numberOfCols = 0;
	int lines = 0;
	int cols = 0;

	fseek( m_file, 0L, SEEK_SET );

	wstring *buffer = new wstring( L"" );

	while( (readln( buffer )) != WEOF )
	{
		cols = countWordsInLine( buffer );

		if( cols > m_numberOfCols )
			m_numberOfCols = cols;

		lines++;
	}
	
	if( lines > m_numberOfLines )
		m_numberOfLines = lines;

	
	delete buffer;
}

int WTextFile::countWordsInLine( wstring *theString )
{
	int wordcounter = 0;
	int length = theString->length();
	int currentPos = 0;

	while (currentPos < length)
	{
		while ((checkWordDelimiter (theString->at( currentPos ))) && (++currentPos < length ))
		{
		}

		currentPos--;

		while ( ( ++currentPos < length ) && (!checkWordDelimiter( theString->at(currentPos) ) ) )
		{
		}
		
		wordcounter++;
	}

	return wordcounter;
}

int WTextFile::returnNumberOfLines( )
{
	return m_numberOfLines;
}

int WTextFile::returnNumberOfColumns( )
{
	return m_numberOfCols;
}

void WTextFile::showWordArray(void)
{
	FILE *file = _wfopen (L"wordArray.txt", L"w");

	for (int line = 0; line < m_numberOfLines; line++)
	{
		int i=0;
		while ( *m_wordArray[line][i] != L"\r" )
		{
			wstring *debug = m_wordArray[line][i];
			fwprintf( file, m_wordArray[line][i]->c_str() );
			fwprintf( file, L" " );
			i++;
		}
		fwprintf( file, L"\n" );
	}

	fclose( file );
}

void WTextFile::deleteRow( int rowNumber )
{
	if( m_numberOfLines > 1 )
	{
		wstring term = L"\r";

		for( int row = rowNumber + 1; row < m_numberOfLines; row++ )
		{
			int column;
			
			for( column = 0; column < m_numberOfCols; column++ )
			{
				*m_wordArray[row-1][column] = *m_wordArray[row][column];
			}

			*m_wordArray[row-1][column] = term;
		}

		m_numberOfLines--;
	}
}

void WTextFile::deleteColumn( int columnNumber )
{
	if( m_numberOfCols > 1 )
	{
		wstring term = L"\r";

		int row, column;

		for( column = columnNumber + 1; column < m_numberOfCols; column++ )
		{	
			for( row = 0; row < m_numberOfLines; row++ )
			{
				*m_wordArray[row][column-1] = *m_wordArray[row][column];
			}
		}

		for( row = 0; row < m_numberOfLines; row++ )
			*m_wordArray[row][column-1] = term;

		m_numberOfCols--;
	}
}

void WTextFile::findReplace( wstring origString, wstring replaceString )
{
	CString origStringC( origString.c_str( ) );
	CString replaceStringC( replaceString.c_str( ) );

	
	for( int row = 0; row < m_numberOfLines; row++ )
	{
		for( int column = 0; column < m_numberOfCols; column++ )
		{
			wstring arrayString = *m_wordArray[row][column];
			CString arrayStringC( arrayString.data( ), arrayString.length( ) );

			arrayStringC.Replace( origStringC, replaceStringC );

			*m_wordArray[row][column] = CStringW( arrayStringC );
		}
	}
}

void WTextFile::findReplace( CString origString, CString replaceString )
{
	for( int row = 0; row < m_numberOfLines; row++ )
	{
		for( int column = 0; column < m_numberOfCols; column++ )
		{
			wstring arrayString = *m_wordArray[row][column];
			CString arrayStringC( arrayString.data( ), arrayString.length( ) );

			arrayStringC.Replace( origString, replaceString );

			*m_wordArray[row][column] = CStringW( arrayStringC );
		}
	}
}